from django.conf import settings
from django.shortcuts import render, redirect
from django.views.generic import FormView
from ..forms import RegisterForm
from .. import utils
from .. import mails
from ..utils import RedirectActiveUser


class Register(RedirectActiveUser, FormView):
    form_class = RegisterForm
    template_name = "django_auth2/register/register.html"

    def form_valid(self, form):
        form.save()

        user = utils.get_user(username=form.cleaned_data['username'])

        if hasattr(settings, 'SEND_ACTIVATION_EMAIL') and settings.SEND_ACTIVATION_EMAIL:
            user.is_active = False
            mails.send_activation_mail(self.request, user)
            return render(
                self.request, template_name='django_auth2/register/email_sended.html'
            )
        else:
            user.is_active = True
            user.save()
            return redirect('index')

register = Register.as_view()



from django.contrib import auth
from django.http import Http404
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.encoding import force_text
from django.utils.http import urlsafe_base64_decode
from django.views import View
from django.views.generic import FormView, TemplateView

from .. import utils
from .. import mails
from ..tokens import account_activation_token
from ..utils import get_user_model
from .. import forms


class ReSend(FormView):
    form_class = forms.RetrySendEmailForActivateForm
    template_name = 'django_auth2/activate/re_send.html'

    def form_valid(self, form):
        email = form.cleaned_data['email']
        user = utils.get_user(email=email)

        mails.send_activation_mail(self.request, user)

        return render(self.request, template_name='django_auth2/register/email_sended.html')

re_send = ReSend.as_view()


class Activate(View):

    def get(self, request, uidb64, token):

        user = self.get_user(uidb64)

        if not user or user.is_active:
            raise Http404

        if not self.token_valid(user, token):
            raise Http404

        user.is_active = True
        user.save()

        auth.login(request, user) # TODO уточнять; возможно чувака переводить на залогиться
        return redirect(reverse('activation_success'))

    def get_user(self, uidb64):
        try:
            uid = force_text(urlsafe_base64_decode(uidb64))
            return utils.get_user(pk=uid)
        except get_user_model().DoesNotExist:
            return

    def token_valid(self, user, token):
        return account_activation_token.check_token(user, token)

activate = Activate.as_view()


class Success(TemplateView):
    template_name = 'django_auth2/activate/success.html'

success = Success.as_view()



from django.core.urlresolvers import reverse
from django.utils.http import *
from django.template.loader import render_to_string
from django.template import Context, Template

from .tokens import account_activation_token, password_reset_token_henerator
from . import tasks


def send_reset_password_mail(request, user):    # TODO: добавить анатацию типов
    uidb64 = urlsafe_base64_encode(force_bytes(user.pk))
    token = password_reset_token_henerator.make_token(user)
    host = request.get_host()
    reset_url = reverse('password_reset_confirm', args=[uidb64, token])

    if request.is_secure():
        scheme = 'https'
    else:
        scheme = 'http'

    url = scheme + '://' + host + reset_url

    context = Context({'url': url})
    message = render_to_string('django_auth2/mail/send_reset_password_mail.html', context)
    subject = 'Смена пароля'

    tasks.send_mail.delay(subject, message, [user.email])


def send_activation_mail(request, user):
    uidb64 = urlsafe_base64_encode(force_bytes(user.pk))
    token = account_activation_token.make_token(user)
    host = request.get_host()
    activation_url = reverse('activate', args=[uidb64, token])

    if request.is_secure():
        scheme = 'https'
    else:
        scheme = 'http'

    url = scheme + '://' + host + activation_url
    context = Context({'url': url})
    message = render_to_string('django_auth2/mail/send_activation_mail.html',
                               context)
    subject = 'Подтверждение аккаунта'

    tasks.send_mail.delay(subject, message, [user.email])



# coding: utf-8
from django.conf.urls import url
from .views import login, register, activate, reset_password, logout


urlpatterns = [
    url(r'^login/$', login.login, name='login'),
    url(r'^register/$', register.register, name='register'),
    url(r'^logout/$', logout.logout, name='logout'),

    url(r'^activate/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$', activate.activate, name='activate'),
    url(r'^activate/re_send/$', activate.re_send, name='activate_re_send'),
    url(r'^activate/success/$', activate.success, name='activation_success'),

    url(r'^password_reset/$', reset_password.password_reset, name='password_reset'),
    url(r'^password_reset/done/$', reset_password.password_reset_done,
        name='password_reset_done'),
    url(
        r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        reset_password.password_reset_confirm, name='password_reset_confirm'),
    url(r'^reset/done/$', reset_password.password_reset_complete,
        name='password_reset_complete'),
]



# coding: utf-8
from django.contrib import auth
from django.core.urlresolvers import reverse_lazy
from django.views.generic import FormView

from ..utils import RedirectActiveUser
from ..forms import LoginForm


class Login(RedirectActiveUser, FormView):
    form_class = LoginForm
    template_name = 'django_auth2/login/login.html'
    success_url = reverse_lazy('index')

    def form_valid(self, form):
        auth.login(self.request, form.get_user())
        return super(Login, self).form_valid(form)

login = Login.as_view()



from django.urls import reverse_lazy
from django.views.generic import FormView
from django.contrib.auth import views as auth_views
from .. import utils
from .. import mails
from .. import forms
from ..tokens import password_reset_token_henerator


class PasswordReset(FormView):
    form_class = forms.PasswordResetForm
    template_name = 'django_auth2/reset_password/password_reset_form.html'
    success_url = reverse_lazy('password_reset_done')

    def form_valid(self, form):
        user = utils.get_user(email=form.cleaned_data['email'])
        mails.send_reset_password_mail(self.request, user)
        return super().form_valid(form)


password_reset = PasswordReset.as_view()

# def password_reset(request, **kwargs):
#     return auth_views.password_reset(request,
#                                      template_name='auth2/reset_password/password_reset_form.html',
#                                      email_template_name='auth2/reset_password/password_reset_email.html',
#                                      **kwargs)


def password_reset_done(request, **kwargs):
    return auth_views.password_reset_done(
        request, template_name='django_auth2/reset_password/password_reset_done.html',
        **kwargs
    )


def password_reset_confirm(request, **kwargs):
    return auth_views.password_reset_confirm(
        request,
        set_password_form=forms.SetPasswordForm,
        token_generator=password_reset_token_henerator,
        template_name='django_auth2/reset_password/password_reset_confirm.html',
        **kwargs
    )


def password_reset_complete(request, **kwargs):
    return auth_views.password_reset_done(
        request,
        template_name='django_auth2/reset_password/password_reset_complete.html',
        **kwargs)



from django.contrib import auth
from django.shortcuts import redirect
from django.urls import reverse


def logout(request):
    redirect_to = reverse('login')
    if request.user.is_authenticated():
        auth.logout(request)

    return redirect(redirect_to)



from django.contrib.auth.tokens import PasswordResetTokenGenerator as DjangoPasswordResetTokenGenerator


class PasswordResetTokenGenerator(DjangoPasswordResetTokenGenerator):
    key_salt = 'fEqUvMDIIY1Yh70GbH9u7e8Fyv716xSs' # IN SETTINGS

password_reset_token_henerator = PasswordResetTokenGenerator()


class AccountActivationTokenGenerator(DjangoPasswordResetTokenGenerator):
    key_salt = 'fEqUvMDIIY1Yh10GbH9u7e8Fyv716xSs'

account_activation_token = AccountActivationTokenGenerator()



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import redirect
from django.urls import reverse


def get_user(**kwargs):
    return get_user_model().objects.get(**kwargs)


def get_user_model():
    if hasattr(settings, 'AUTH_USER_MODEL'):
        app, model = settings.AUTH_USER_MODEL.split('.')
        model = model.lower()
        content_type = ContentType.objects.get(app_label=app, model=model)
        return content_type.model_class()
    else:
        from django.contrib.auth.models import User as DjangoUser
        return DjangoUser


class RedirectActiveUser(object):

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return redirect(reverse('index'))

        return super(RedirectActiveUser, self).dispatch(request, *args,
                                                        **kwargs)



from __future__ import absolute_import # for python 2.7

from django.conf import settings
from django.core.mail import send_mail as django_send_mail

try:
    from celery.task import task

    @task
    def send_mail(subject, message, to_emails):
        django_send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, to_emails)

except ImportError:

    def func_add_delay(func):
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        wrapper.delay = func
        return wrapper

    @func_add_delay
    def send_mail(subject, message, to_emails):
        django_send_mail(subject, message, settings.DEFAULT_FROM_EMAIL,
                         to_emails)

        send_mail.delay = send_mail


import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()

setup(
    name='django_auth2',
    version='0.0.8',
    description='Login, Registration, Reset password, ActivateUser',
    long_description=README,
    author='nick1994209',
    author_email='nick1994209@gmail.com',
    url='https://github.com/Nick1994209/django-auth2/',
    license='MIT',
    packages=find_packages(),
    # packages=['django_auth2'],
    include_package_data=True,
    install_requires=[
        'django',
        # 'celery' if your need
    ]
)



# -*- coding:utf-8 -*-
from django.contrib.auth import authenticate
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.forms import SetPasswordForm as DjangoSetPasswordForm
from django import forms
from . import utils


class LoginForm(AuthenticationForm):
    error_messages = {
        'invalid_login': "Пожалуйста введите корректный логин или пароль",
        'inactive': "Аккаунт не активен",
    }

    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)
        # self.fields['username'].label = "Логин"
        # self.fields['password'].label = "Пароль"

    def clean(self):
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')

        if username and password:
            user_kwargs = {utils.get_user_model().USERNAME_FIELD: username}
            try:
                user = utils.get_user(**user_kwargs)

                if user.is_active == False:
                    raise forms.ValidationError(
                        "Аккаунт не активен, хотите выслать подтверждение еще раз?",
                        code='invalid_account',
                        # params={'username': self.username_field.verbose_name},
                    )
            except utils.get_user_model().DoesNotExist:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    # params={'username': self.username_field.verbose_name},
                )

            self.user_cache = authenticate(password=password, **user_kwargs)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    # params={'username': self.username_field.verbose_name},
                )

        return self.cleaned_data


class RegisterForm(forms.ModelForm):
    password1 = forms.CharField(label=("Пароль"),
                                strip=False,
                                widget=forms.PasswordInput)
    password2 = forms.CharField(label=("Повторите пароль"),
                                widget=forms.PasswordInput,
                                strip=False,
                                help_text=("Пароли должны совподать"))

    # def __init__(self, **qwargs):
    #     super(RegisterForm, self).__init__(**qwargs)
        # self.fields['username'].label = "Логин"
        # self.fields['username'].error_messages['required'] = 'Требуется'

    class Meta:
        model = utils.get_user_model()
        fields = utils.get_user_model().REQUIRED_FIELDS + [utils.get_user_model().USERNAME_FIELD]

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                u'Пароли не совпадают',
                code='password_mismatch',
            )
        return password2

    def save(self, commit=True):
        user = super(RegisterForm, self).save(commit=False)
        for field, value in self.cleaned_data.items():
            setattr(user, field, value)
        # user.set_password(self.cleaned_data["password1"])
        # user.email = self.cleaned_data['email']
        user_model = utils.get_user_model()
        # user = user_model(**self.cleaned_data)
        user.set_password(self.cleaned_data["password1"])

        if commit:
            user.save()
        return user


class RetrySendEmailForActivateForm(forms.Form):
    email = forms.EmailField(required=True, label='Email')

    def clean_email(self):
        email = self.cleaned_data['email']

        try:
            user = utils.get_user(email=email)

            if user.is_active == True:
                raise forms.ValidationError(
                    "Аккаунт активен",
                    code='account_is_active',
                )

        except utils.get_user_model().DoesNotExist:
            raise forms.ValidationError(
                'Пользователя не найдено',
            )
        return email


class PasswordResetForm(forms.Form):
    email = forms.EmailField(required=True, label='Email')

    def clean(self):
        email = self.cleaned_data['email']

        try:
            user = utils.get_user(email=email)

            if user.is_active == False:
                raise forms.ValidationError(
                    "Аккаунт не активен, хотите выслать подтверждение еще раз?",
                    code='account_not_active',
                )

        except utils.get_user_model().DoesNotExist:
            raise forms.ValidationError(
                'Пользователя не найдено',
            )


class SetPasswordForm(DjangoSetPasswordForm):

    def __init__(self, *args, **kwargs):
        super(SetPasswordForm, self).__init__(*args, **kwargs)
        self.fields['new_password1'].label = 'Новый пароль'
        self.fields['new_password2'].label = 'Повторите пароль'
